// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __CIRCULARBUFFER_H
#define __CIRCULARBUFFER_H

#include <string>

class CircularBuffer {
public:
  CircularBuffer(int initSize, int maxSize);
  virtual ~CircularBuffer();

  // returns number of bytes waiting to be sent
  int getUsed() const {
    return used;
  }

  // returns number of free bytes remaining in the buffer
  int getFree() const {
    return maxSize - used;
  }

  int getMaxSize() const {
    return maxSize;
  }

  bool hasOverflowed() const {
    return overflow;
  }

  void clearOverflowed() {
    overflow = false;
  }

  bool write(const char *data, int size);
  bool write(const std::string &data);

  bool read(char *data, int size);
  bool read(std::string &data, int count);
  bool readLine(std::string &data, char ch = '\n');

  void discard(int size);

protected:
  void getSections(char *&buffer1, int &count1, 
		   char *&buffer2, int &count2);

  void setOverflowed() {
    overflow = true;
  }

private:
  char *buffer;
  int start, size, maxSize, used;
  bool overflow;

  void grow();
};

#endif // __CIRCULARBUFFER_H
