/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "config.h"
#include "psfilter.h"

#define IPACKS 2048
#define TS_SIZE 188

PSFilter::PSFilter(Source *source)
  : source(source)
{
  init_ipack(&packer, IPACKS, output, 1);
  packer.data = this;
}

PSFilter::~PSFilter()
{
  unsubscribeAll();
  free_ipack(&packer);
}

// Static function for use by the ipack code
void PSFilter::output(uint8_t *data, int size, void *priv)
{
  PSFilter *d = static_cast<PSFilter *>(priv);
  d->sendData(data, (unsigned) size);
}

void PSFilter::unsubscribe(Sink *s) 
{
  Source::unsubscribe(s);
  if (getNumSubscribers() == 0) {
    source->unsubscribe(this);
    reset_ipack(&packer);
  }
}

void PSFilter::notifyUnsubscribe(Source *)
{
  // Since our source is no longer available
  // we should unsubscribe the clients.
  unsubscribeAll();
}

bool PSFilter::subscribe(Sink *s)
{
  if (getNumSubscribers() == 0) {
    if (!source->subscribe(this))
      return false;
  } 
  return Source::subscribe(s);
}

void PSFilter::receiveData(unsigned char *data, unsigned size)
{
  // adapted from my_ts_to_ps( uint8_t* buf, uint16_t pida, uint16_t pidv)
  // in dvbstream.c (Dave Chapman)

  ipack *p = &packer;
  uint8_t off = 0;
  uint8_t *buf = (uint8_t *) data;

  if (!(buf[3]&0x10)) // no payload?
    return;

  if ( buf[1]&0x40) {
    if (p->plength == MMAX_PLENGTH-6){
      p->plength = p->found-6;
      p->found = 0;
      send_ipack(p);
      reset_ipack(p);
    }
  }

  if ( buf[3] & 0x20) {  // adaptation field?
    off = buf[4] + 1;
  }
        
  instant_repack(buf+4+off, TS_SIZE-4-off, p);
}
